LIBRARY libnfs
EXPORTS
mount_free_export_list
mount_getexports
mount_getexports_async
nfs_find_local_servers
free_nfs_srvr_list
nfs_access
nfs_access_async
nfs_chdir
nfs_chmod
nfs_chmod_async
nfs_chown
nfs_chown_async
nfs_close
nfs_close_async
nfs_closedir
nfs_creat
nfs_creat_async
nfs_create
nfs_create_async
nfs_destroy_context
nfs_fchmod
nfs_fchmod_async
nfs_fchown
nfs_fchown_async
nfs_fcntl
nfs_fcntl_async
nfs_fstat
nfs_fstat_async
nfs_fstat64
nfs_fstat64_async
nfs_fsync
nfs_fsync_async
nfs_ftruncate
nfs_ftruncate_async
nfs_get_error
nfs_get_fd
nfs_get_readmax
nfs_get_writemax
nfs_getcwd
nfs_get_version
nfs_get_timeout
nfs_init_context
nfs_link
nfs_link_async
nfs_lockf
nfs_lockf_async
nfs_lseek
nfs_lseek_async
nfs_mkdir
nfs_mkdir_async
nfs_mkdir2
nfs_mkdir2_async
nfs_mknod
nfs_mknod_async
nfs_mount
nfs_mount_async
nfs_mt_service_thread_start
nfs_mt_service_thread_stop
nfs_open
nfs_open_async
nfs_open2
nfs_open2_async
nfs_opendir
nfs_opendir_async
nfs_parse_url_full
nfs_parse_url_dir
nfs_parse_url_incomplete
nfs_destroy_url
nfs_pagecache_invalidate
nfs_pread
nfs_pread_async
nfs_pwrite
nfs_pwrite_async
nfs_read
nfs_read_async
nfs_readdir
nfs_readlink
nfs_readlink_async
nfs_readlink2
nfs_rename
nfs_rename_async
nfs_rmdir
nfs_rmdir_async
nfs_service
nfs_set_auth
nfs_set_autoreconnect
nfs_set_debug
nfs_set_dircache
nfs_set_gid
nfs_set_pagecache
nfs_set_pagecache_ttl
nfs_set_readahead
nfs_set_readmax
nfs_set_tcp_syncnt
nfs_set_timeout
nfs_set_uid
nfs_set_version
nfs_set_writemax
nfs_stat
nfs_stat_async
nfs_stat64
nfs_stat64_async
nfs_statvfs
nfs_statvfs_async
nfs_statvfs64
nfs_statvfs64_async
nfs_symlink
nfs_symlink_async
nfs_truncate
nfs_truncate_async
nfs_umount
nfs_umount_async
nfs_unlink
nfs_unlink_async
nfs_utime
nfs_utime_async
nfs_utimes
nfs_utimes_async
nfs_which_events
nfs_write
nfs_write_async
nfsstat3_to_errno
nfsstat3_to_str
nfsstat4_to_errno
nfsstat4_to_str
nfs3_getacl
nfs3_getacl_async
nfs4_set_client_name
nfs4_set_verifier
nfs4_acl_free
nfs4_getacl
nfs4_getacl_async
win32_poll
rpc_connect_async
rpc_connect_port_async
rpc_connect_program_async
rpc_destroy_context
rpc_disconnect
rpc_get_error
rpc_get_fd
rpc_init_context
rpc_init_server_context
rpc_pmap2_null_async
rpc_pmap2_getport_async
rpc_pmap2_set_async
rpc_pmap2_unset_async
rpc_pmap2_dump_async
rpc_pmap2_callit_async
rpc_pmap3_null_async
rpc_pmap3_set_async
rpc_pmap3_unset_async
rpc_pmap3_getaddr_async
rpc_pmap3_dump_async
rpc_pmap3_callit_async
rpc_pmap3_gettime_async
rpc_pmap3_uaddr2taddr_async
rpc_pmap3_taddr2uaddr_async
rpc_mount_null_async
rpc_mount_mnt_async
rpc_mount_dump_async
rpc_mount_umnt_async
rpc_mount_umntall_async
rpc_mount_export_async
rpc_mount1_null_async
rpc_mount1_mnt_async
rpc_mount1_dump_async
rpc_mount1_umnt_async
rpc_mount1_umntall_async
rpc_mount1_export_async
rpc_mount3_null_async
rpc_mount3_mnt_async
rpc_mount3_dump_async
rpc_mount3_umnt_async
rpc_mount3_umntall_async
rpc_mount3_export_async
rpc_nfsacl_null_async
rpc_nfsacl_getacl_async
rpc_nfsacl_setacl_async
rpc_nfs_null_async
rpc_nfs_getattr_async
rpc_nfs_pathconf_async
rpc_nfs_lookup_async
rpc_nfs_access_async
rpc_nfs_read_async
rpc_nfs_write_async
rpc_nfs_commit_async
rpc_nfs_setattr_async
rpc_nfs_mkdir_async
rpc_nfs_rmdir_async
rpc_nfs_create_async
rpc_nfs_mknod_async
rpc_nfs_remove_async
rpc_nfs_readdir_async
rpc_nfs_readdirplus_async
rpc_nfs_fsstat_async
rpc_nfs_fsinfo_async
rpc_nfs_readlink_async
rpc_nfs_symlink_async
rpc_nfs_rename_async
rpc_nfs_link_async
rpc_nfs2_null_async
rpc_nfs2_getattr_async
rpc_nfs2_setattr_async
rpc_nfs2_lookup_async
rpc_nfs2_readlink_async
rpc_nfs2_read_async
rpc_nfs2_write_async
rpc_nfs2_create_async
rpc_nfs2_remove_async
rpc_nfs2_rename_async
rpc_nfs2_link_async
rpc_nfs2_symlink_async
rpc_nfs2_mkdir_async
rpc_nfs2_rmdir_async
rpc_nfs2_readdir_async
rpc_nfs2_statfs_async
rpc_nfs3_null_async
rpc_nfs3_getattr_async
rpc_nfs3_pathconf_async
rpc_nfs3_lookup_async
rpc_nfs3_access_async
rpc_nfs3_read_async
rpc_nfs3_write_async
rpc_nfs3_commit_async
rpc_nfs3_setattr_async
rpc_nfs3_mkdir_async
rpc_nfs3_rmdir_async
rpc_nfs3_create_async
rpc_nfs3_mknod_async
rpc_nfs3_remove_async
rpc_nfs3_readdir_async
rpc_nfs3_readdirplus_async
rpc_nfs3_fsstat_async
rpc_nfs3_fsinfo_async
rpc_nfs3_readlink_async
rpc_nfs3_symlink_async
rpc_nfs3_rename_async
rpc_nfs3_link_async
rpc_nfs4_compound_async
rpc_nfs4_null_async
rpc_nlm4_null_async
rpc_nlm4_test_async
rpc_nlm4_lock_async
rpc_nlm4_cancel_async
rpc_nlm4_unlock_async
rpc_nsm1_null_async
rpc_nsm1_stat_async
rpc_nsm1_mon_async
rpc_nsm1_unmon_async
rpc_nsm1_unmonall_async
rpc_nsm1_simucrash_async
rpc_nsm1_notify_async
rpc_null_async
rpc_register_service
rpc_rquota1_null_async
rpc_rquota1_getquota_async
rpc_rquota1_getactivequota_async
rpc_send_reply
rpc_service
rpc_set_fd
rpc_set_gid
rpc_set_uid
rpc_which_events
